<?php

namespace App\Service;

use \Datetime;
use Dompdf\Dompdf;
use Dompdf\Options;
use Twig\Environment;
use Twig\Loader\FilesystemLoader;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xls;

class Reports 
{
  private $util;
  private $orderService;

  public function __construct(\App\Util\Util $util, \App\Service\Order $orderService) {
    $this->util = $util;
    $this->orderService = $orderService;
  }

  public function exportPDF($params) {   
    $title = $params['title'];
    $order = $this->orderService->history($params['paramsFilter']);
    $formattedOrders = $this->formatOrder($order);
    $html = $this->formatContent($title, $formattedOrders, $params['visibleColumns']);

    $options = new Options();
    $options->set('isHtml5ParserEnabled', true);
    $options->set('isPhpEnabled', true);
    $options->setIsRemoteEnabled(true);

    $dompdf = new Dompdf($options);
    $dompdf->loadHtml($html);
    $dompdf->setPaper('A4', 'landscape');
    $dompdf->render();
    $pdfOutput = $dompdf->output();

    return $pdfOutput;
  }

  public function exportXLS($params) {
    $title = $params['title'];
    $headers = $params['visibleColumns'];
    $order = $this->orderService->history($params['paramsFilter']);
    $formattedOrders = $this->formatOrder($order);

    $spreadsheet = new Spreadsheet();
    $sheet = $spreadsheet->getActiveSheet();

    $row = 1;
    $column = 'A';
    $headerMap = [];
    foreach ($headers as $header) {
      $headerKey = $header['key'];
      $headerLabel = $header['label'];
      $headerMap[$headerKey] = $headerLabel;
      $sheet->setCellValue($column . $row, $headerLabel);

      $style = array(
        'font' => array('bold' => true),
        'alignment' => array('horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER),
        'fill' => array('fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID, 'startColor' => array('rgb' => 'DDDDDD')),
        'borders' => array(
          'bottom' => array('style' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN),
      ),
    );
      $sheet->getStyle($column . $row)->applyFromArray($style);
      $sheet->getColumnDimension($column)->setAutoSize(true);

      $column++;
    }
    $row++;

    foreach ($formattedOrders as $rowIndex => $items) {
      $column = 'A';
      foreach ($headers as $headerObj) {
        $headerKey = $headerObj['key'];
        $cellValue = isset($items[$headerKey]) ? $items[$headerKey] : '';
        $sheet->setCellValue($column . $row, $cellValue);

        $style = $sheet->getStyle($column . $row);
        $style->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);  

        if ($rowIndex % 2 == 1) {
            $style->getFill()->setFillType(\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID);
            $style->getFill()->getStartColor()->setARGB('F2F2F2');
        }
          $column++;
        }
        $row++;
      }

      $writer = new Xls($spreadsheet);
      $tempFolder = $this->getTemFolderPath();
      $filePath = $tempFolder . DIRECTORY_SEPARATOR . $title . '.xls';

      $writer->save($filePath);

      if (!file_exists($filePath)) {
          throw new \Exception('Falha ao salvar o arquivo XLS.');
      }

      $fileContent = file_get_contents($filePath);
      $base64Content = base64_encode($fileContent);
      unlink($filePath);
      return $base64Content;
  }

  private function getTemFolderPath() {
    return sys_get_temp_dir();
  }

  private function formatOrder($order) {
    $formattedOrders = array();
    foreach ($order as $items) {
        $formattedOrderItem = array(
          'IDSTCOMANDA'     => $this->util->formatStatus($items['IDSTCOMANDA'], $items['IDSINCMSDE']),
          'IDORGCMDVENDA'   => $this->util->commandSourceDictionary($items['IDORGCMDVENDA']),
          'IDSINCMSDE'      => $this->util->formatSynced($items['IDSINCMSDE']),
          'VRPEDIDO'        => $this->util->formatMoney($items['VRPEDIDO']),
          'VRTOTALPRODUTOS' => $this->util->formatMoney($items['VRTOTALPRODUTOS']),
          'VRDESCOMANDA'    => $this->util->formatMoney($items['VRDESCOMANDA']),
          'ACRESCIMO'       => $this->util->formatMoney($items['ACRESCIMO']),
          'VRACRCOMANDA'    => $this->util->formatMoney($items['VRACRCOMANDA']),
        );
      $formattedOrders[] = array_merge($items, $formattedOrderItem);
    }
    return $formattedOrders;
  }

  private function formatContent($title, $formattedOrders, $visibleColumnLabels) {
    $loader = new FilesystemLoader(__DIR__ . '/../../templates');
    $twig = new Environment($loader);
    $template = $twig->load('reports.twig');

    $host = '';
    if (isset($_SERVER['HTTP_ORIGIN']) && $_SERVER['HTTP_ORIGIN']) {
      $host = $_SERVER['HTTP_ORIGIN'];
    } else {
      $host = $this->getKey('origin');
    }

    $formattedContent = $template->render([
      'imgLogo' => $host . "/assets/retail-control.png",
      'title' => $title,
      'datetime' => (new DateTime())->format('d/m/Y H:i:s'),
      'order' => $formattedOrders,
      'visibleColumn' => $visibleColumnLabels
    ]);

    return $formattedContent;
  }

  private function getKey(String $key, $defaultValue = "") {
    $loweredKey = strtolower($key);
    $allHeaders = $this->getAllHeaders();
    $responseValue = isset($allHeaders[$loweredKey]) ? $allHeaders[$loweredKey] : $defaultValue;
    return $responseValue;
  }

  private function getAllHeaders() {
    $allHeaders = getallheaders();
    if (!empty($allHeaders)) {
      $allHeaders = array_change_key_case($allHeaders);
      return $allHeaders;
    } else {
      return [];
    }
  }
}